// large.cpp: Multi-precision integer arithmetic,
//    based on STL.
// From: Ammeraal, L. (1997) STL for C++ Programmers,
//       Chichester: John Wiley.
#include <iostream>
#include <strstream>
#include <iomanip>
#include <stdlib.h>
#include <limits.h>
#include <string>
#include <ctype.h>
#include "large.h"

const uint uintmax = UINT_MAX;
const int wLen = sizeof(uint) * 8;  // Number of bits
const int hLen = wLen/2;
const uint rMask = (1 << hLen) - 1;
const uint lMask = uintmax - rMask;
const uint lBit = uintmax - (uintmax >> 1);

// Either insert zeros or delete some elements
// at the end, that is, in high-order positions:
void large::SetLen(int LenNew)
{  int LenOld = P.size();
   if (LenNew > LenOld) 
      P.insert(P.end(), LenNew - LenOld, 0); else
   if (LenNew < LenOld)
      P.erase(P.begin()+LenNew, P.end());
}

// Remove any leading zeros (at the end of vector P):
void large::reduce()
{  int L = P.size();
   while (L > 0 && P[L-1] == 0) L--;
   P.erase(P.begin() + L, P.end());
   if (L == 0) neg = 0;
}

// Some constructors:
large::large(const char *str)
{  bool Neg = *str == '-';
   int i = (Neg ? 1 : 0);
   *this = 0;
   while (str[i]) 
      *this = *this * 10 + (str[i++] - '0');
   neg = Neg;
   reduce();
}

large::large(int i)
{  if (i > 0) P.push_back(i); else
   if (i < 0) P.push_back(-i);
   neg = i < 0;
}

large::large(long L)
{  neg = L < 0;
   unsigned long UL = neg ? -L : L;
   if (UL != 0)
   {
#if LONG_MAX == INT_MAX
      // sizeof(long) == sizeof(int):
      P.push_back(uint(UL)); 
#else   
      // sizeof(long) > sizeof(int):
      while (UL != 0)
      {  P.push_back(uint(UL & UINT_MAX));
         UL >>= wLen;
      }  
#endif
   }
}
large::large(uint i)
{  neg = 0; 
   if (i) P.push_back(i);
}

// Operator + defined in terms of +=, and so on:
large operator+(large x, const large &y){return x+=y;}
large operator-(large x, const large &y){return x-=y;}
large operator*(large x, const large &y){return x*=y;}
large operator/(large x, const large &y){return x/=y;}
large operator%(large x, const large &y){return x%=y;}
large operator<<(large u, uint k){return u <<= k;}
large operator>>(large u, uint k){return u >>= k;}

large large::operator-()const
{  large v = *this;
   if (v.P.size()) v.neg = !v.neg;
   return v;
}

large &large::operator+=(const large &y)
{  if (neg != y.neg) return *this -= -y;
   int i, Ly = y.P.size();
   uint d, carry = 0;
   // Result length = max. operand length + 1:
   SetLen(max(Ly, (int)P.size()) + 1);
   int L = P.size();
   for (i=0; i<L; i++)
   {  if (i >= Ly && carry == 0) break;
      d = P[i] + carry;
      // Reset 'carry' to zero unless last
      // addition caused another carry:
      carry = d < carry;
      if (i < Ly)
      {  P[i] = d + y.P[i];
         // Only if 'carry' is now zero (and d > 0)
         // can last addition cause a carry:
         if (P[i] < d) carry = 1;
      }  else P[i] = d;
   }
   reduce();
   return *this;
}

large &large::operator-=(const large &y)
{  if (neg != y.neg) return *this += -y;
   if (!neg && y > *this || neg && y < *this)
      return *this = -(y - *this);
   int i, borrow = 0, Ly = y.P.size(), L = P.size();
   uint d;
   for (i=0; i<L; i++)
   {  if (i >= Ly && borrow == 0) break;
      d = P[i] - borrow;
      borrow = d > P[i];
      if (i < Ly)
      {  P[i] = d - y.P[i];
         if (P[i] > d) borrow = 1;
      }  else P[i] = d;
   }
   reduce();
   return *this;
}

large &large::operator*=(int y)
{  bool Neg = y < 0;
   if (Neg) y = -y;
   *this *= uint(y);
   if (Neg) 
      neg = !neg;
   return *this;
}

large &large::operator*=(uint y)
{  int len0 = P.size(), i;
   uint Hi, Lo, dig = P[0], nextdig = 0;
   SetLen(len0 + 1);
   for (i=0; i<len0; i++)
   {  // Compute double-digit product dig * y;
      // result is (Hi, Lo):
      DDproduct(dig, y, Hi, Lo);
      P[i] = Lo + nextdig;
      dig = P[i+1];
      nextdig = Hi + (P[i] < Lo);
   }
   P[i] = nextdig;
   reduce();
   return *this;
}

large &large::operator*=(large y)
{  int L = P.size(), Ly = y.P.size();
   if (L == 0 || Ly == 0) return *this = 0;
   bool DifSigns = neg != y.neg;
   if (L + Ly == 2) 
   // L = Ly = 1: single- or double-digit product:
   {  uint a = P[0], b = y.P[0];
      P[0] = a * b; // Try single-digit product
      if (P[0] / a != b)
      {  P.push_back(0);
         DDproduct(a, b, P[1], P[0]);
         reduce();
      }
      neg = DifSigns;
      return *this;
   }
   if (L == 1)  //  && Ly > 1
   {  uint digit = P[0]; 
      *this = y;       // Swap operands and
      *this *= digit;  // call operator*=(uint y)
   }  else
   if (Ly == 1) //  && L > 1
   {  *this *= y.P[0]; // Call operator*=(uint y)
   }  else
   // Both operand lengths L and Ly greater than 1:
   {  int lenProd = L + Ly, i, jA, jB;
      uint sumHi = 0, sumLo, Hi, Lo,
      sumLoOld, sumHiOld, carry=0;
      large x = *this;
      SetLen(lenProd); // Give *this length lenProd

      for (i=0; i<lenProd; i++)
      {  sumLo = sumHi; sumHi = carry; carry = 0;
         int max_jA = min(i, L-1); 
         // jA <= i ensures jB >= 0
         // jA < L since only L digits in *this
         for (jA=max(0, i+1-Ly); jA<=max_jA; jA++)
         // jA > i - Ly ensures jB < Ly
         {  jB = i - jA;
            // Another digit product contributing to
            // position i (= jA + jB) of number product:
            DDproduct(x.P[jA], y.P[jB], Hi, Lo);
            sumLoOld = sumLo; sumHiOld = sumHi;
            sumLo += Lo;
            if (sumLo < sumLoOld) 
               sumHi++;
            sumHi += Hi; 
            carry += (sumHi < sumHiOld);
         }
         P[i] = sumLo;
      }
   }
   reduce();
   neg = DifSigns;
   return *this;
}

large &large::operator/=(const large &divisor)
{  large r;
   // Divide *this by divisor, giving quotient in
   // *this; 0 indicates that remainder in r
   // need not be correct:
   divide(divisor, *this, r, 0);
   return *this;
}

large &large::operator%=(const large &divisor)
{  large q;
   // Divide *this by divisor, giving quotient in
   // q and remainder in *this; 1 indicates that 
   // this remainder has to be correct:
   divide(divisor, q, *this, 1);
   return *this;
}

large &large::operator<<=(uint k)
{  int q = k / wLen; // Number of full words.
   if (q) 
   {  int i;         // Increase length by q:
      SetLen(P.size() + q);

      // Shift *this by q words to the left:
      for (i=P.size()-1; i>=0; i--)
         P[i] = (i < q ? 0 : P[i - q]);
      k %= wLen;     // k is now the remaining
   }                 // number of shift positions.

   if (k)  // 0 < k < wLen:
   {  int k1 = wLen - k;
      uint mask = (1 << k) - 1;
      // mask: 00..011..1 (k one-bits)
      SetLen(P.size() + 1);

      // Each P[i] shifts k positions to the left
      // and is then ORed with k leftmost bits
      // of right neighbor P[i-1]:
      for (int i=P.size()-1; i>=0; i--)
      {  P[i] <<= k;
         if (i > 0) 
            P[i] |= (P[i-1] >> k1) & mask;
      }
   }

   reduce();
   return *this;
}
large &large::operator>>=(uint k)  // Analogous
{  int q = k / wLen, L = P.size(); // to <<=
   if (q >= L){*this = 0; return *this;}
   if (q)
   {  for (int i=q; i<L; i++) P[i-q] = P[i];
      SetLen(L - q);
      k %= wLen;
      if (k == 0){reduce(); return *this;}
   }
   int n = P.size() - 1, k1 = wLen - k;
   uint mask = (1 << k) - 1;
   for (int i=0; i<=n; i++)
   {  P[i] >>= k;
      if (i < n) P[i] |= ((P[i+1] & mask) << k1);
   }
   reduce();
   return *this;
}

// compare returns: negative if *this < y,
// zero if *this == y, and positive if *this > y.
int large::compare(const large &y)const
{  if (neg != y.neg) return y.neg - neg;
   int code = 0, L = P.size(), Ly = y.P.size();
   if (L == 0 || Ly == 0) code = L - Ly; else
   if (L < Ly) code = -1; else
   if (L > Ly) code = +1; else
   for (int i = L - 1; i >= 0; i--)
   {  if (P[i] > y.P[i]) {code = 1; break;} else
      if (P[i] < y.P[i]) {code = -1; break;}
   }
   return neg ? -code : code;
}

// Double-digit product (Hi, Lo) = A * B:
void large::DDproduct(uint A, uint B, 
                      uint &Hi, uint &Lo)const
{  uint hiA = A >> hLen, loA = A & rMask,
        hiB = B >> hLen, loB = B & rMask,
        mid1, mid2, old;
   Lo = loA * loB; Hi = hiA * hiB;
   mid1 = loA * hiB; mid2 = hiA * loB;
   old = Lo; 
   Lo += mid1 << hLen; 
   Hi += (Lo < old) + (mid1 >> hLen);
   old = Lo; 
   Lo += mid2 << hLen; 
   Hi += (Lo < old) + (mid2 >> hLen);
}
// Double-digit value (A, B) is divided by d:

uint large::DDquotient(uint A, uint B, uint d)const
{  uint left, middle, right, qHi, qLo, x, dLo1,
      dHi = d >> hLen, dLo = d & rMask;
   qHi = A/(dHi + 1);
   // This initial guess of qHi may be too small.
   middle = qHi * dLo; 
   left = qHi * dHi;
   x = B - (middle << hLen);
   A -= (middle >> hLen) + left + (x > B); B = x;
   dLo1 = dLo << hLen;

   // Increase qHi if necessary:
   while (A > dHi || (A == dHi && B >= dLo1))
   {  x = B - dLo1; 
      A -= dHi + (x > B); 
      B = x; 
      qHi++;
   }
   qLo = ((A << hLen) | (B >> hLen))/(dHi + 1);
   // This initial guess of qLo may be too small.
   right = qLo * dLo; middle = qLo * dHi;
   x = B - right; 
   A -= (x > B); 
   B = x;
   x = B - (middle << hLen); 
   A -= (middle >> hLen) + (x > B);
   B = x;
   // Increase qLo if necessary:
   while (A || B >= d)
   {  x = B - d; 
      A -= (x > B); 
      B = x; 
      qLo++;
   }
   uint result = (qHi << hLen) + qLo;
   return result == 0 && qHi > 0 ? uintmax : result;
}

// Subtract multiple q * b from a, where a and b
// are values of n digits. The remainder a - q * b
// will be less than b and must not be negative.
// The latter condition may require q to be 
// decreased by 1:

void large::subtractmul(uint *a, uint *b, int n, 
   uint &q)const
{  uint Hi, Lo, d, carry = 0;
   int i;
   for (i=0; i<n; i++)
   {  DDproduct(b[i], q, Hi, Lo);
      d = a[i];
      a[i] -= Lo;
      if (a[i] > d) carry++;
      d = a[i + 1];
      a[i + 1] -= Hi + carry;
      carry = a[i + 1] > d;
   }
   if (carry) // q was too large
   {  q--; carry = 0;
      for (i=0; i<n; i++)
      {  d = a[i] + carry;
         carry = d < carry;
         a[i] = d + b[i];
         if (a[i] < d) 
            carry = 1;
      }
      a[n] = 0;
   }
}

// Normalize by shifting denom and num to the left,
// so that the leftmost position of denom is 1;
// both operands of num/denom shift by x bit positions:
bool large::normalize(large &denom, large &num, 
   int &x)const
{  int r = denom.P.size() - 1;
   uint y = denom.P[r]; x = 0;
   while ((y & lBit) == 0){y <<= 1; x++;}
   denom <<= x; num <<= x;
   // Possibly second action according to C. J. Mifsud
   // (see Bibliography):
   if (r > 0 && denom.P[r] < denom.P[r-1])
   {  denom *= uintmax; num *= uintmax;
      return 1;
   }
   return 0;
}

// Undo the effect of normalizing (including Mifsud's
// correction if SecondDone == 1) to obtain the 
// correct remainder:

void large::unnormalize(large &rem, int x, bool 
   SecondDone)const
{  if (SecondDone) rem /= uintmax;
   if (x > 0) rem >>= x; else rem.reduce();
}
// Divide *this by denom, giving quot = num / denom,
// and, if RemDesired == 1, rem = num % denom:
void large::divide(large denom,
   large &quot, large &rem, bool RemDesired)const
{  int L = P.size(), Ld = denom.P.size();
   if (Ld == 0) {cout << "Zero divide.\n"; return;}
   bool QuotNeg = neg ^ denom.neg;
   int i, r, x = 0, n;
   bool RemNeg = neg, SecondDone = false;
   uint q, d;
   large num = *this;
   num.neg = denom.neg = 0;
   if (num < denom)
   {  quot = 0; rem = num; rem.neg = RemNeg; return;
   }
   if (Ld == 1 && L == 1)
   {  quot = uint(num.P[0]/denom.P[0]);
      rem = uint(num.P[0]%denom.P[0]);
      quot.neg = QuotNeg; rem.neg = RemNeg;
      return;
   }  else
   if (Ld == 1 && (denom.P[0] & lMask) == 0)
   {  // Denominator fits into a half word
      uint divisor = denom.P[0], dHi = 0,
         q1, r, q2, dividend;
      quot.SetLen(L);
      for (int i=L-1; i>=0; i--)
      {  dividend = (dHi << hLen) | (P[i] >> hLen);
         q1 = dividend/divisor; r = dividend % divisor;
         dividend = (r << hLen) | (P[i] & rMask);
         q2 = dividend/divisor; 
         dHi = dividend % divisor;
         quot.P[i] = (q1 << hLen) | q2;
      }
      quot.reduce(); rem = dHi;
      quot.neg = QuotNeg; rem.neg = RemNeg;
      return;
   }
   large num0 = num, denom0 = denom;
   SecondDone = normalize(denom, num, x);
   r = denom.P.size() - 1; n = num.P.size() - 1;
   quot.SetLen(n - r);
   int Lq = quot.P.size();
   for (i=Lq-1; i>=0; i--) quot.P[i] = 0;
   rem = num;
   if (rem.P[n] >= denom.P[r])
   {  rem.SetLen(rem.P.size() + 1); 
      n++;
      quot.SetLen(Lq + 1);
   }
   d = denom.P[r];
   for (int k=n; k>r; k--)
   {  q = DDquotient(rem.P[k], rem.P[k-1], d);
      subtractmul(&rem.P[k - r - 1], &denom.P[0], 
         r + 1, q);
      quot.P[k - r - 1] = q;
   }
   quot.reduce(); quot.neg = QuotNeg;
   if (RemDesired)
   {  unnormalize(rem, x, SecondDone); 
      rem.neg = RemNeg;
   }
}

bool operator==(const large &x, const large &y)
{  return x.compare(y) == 0;
}
bool operator<(const large &x, const large &y)
{   return x.compare(y) < 0;
}

bool operator!=(const large& x, const large& y)
{  return x.compare(y) != 0;
}

bool operator>(const large& x, const large& y)
{  return x.compare(y) > 0;
}

// Function num2char converts a large object x to
// its character representation s, in reverse order.
void large::num2char(vector<char> &s) const
{  large x = *this;
   static uint p10 = 1, ip10 = 0;
   if (x.P.size() == 0) s.push_back('0'); else
   {  uint r;
      if (p10 == 1)
      {  while (p10 <= UINT_MAX/10)
         {  p10 *= 10;
            ip10++;
         }
      }  // p10 is the largest possible uint power of 10
         // LP10 = p10 = pow(10, ip10)
      large R, LP10 = p10;  
      bool neg = x.neg;
      do
      {  x.divide(LP10, x, R, 1);
         r = (R.P.size() ? R.P[0] : 0);
         for (uint j=0; j<ip10; j++)
         {  s.push_back(char(r % 10 + '0'));
            r /= 10;
            if (r + x.P.size() == 0) break;
         }
      }  while (x.P.size());
      if (neg) s.push_back('-');
      // s contains string in reverse order
   }
}
ostream &operator<<(ostream &os, const large &x)
{  vector<char> s;
   x.num2char(s);
   vector<char>::reverse_iterator i;
   for (i=s.rbegin(); i != s.rend(); ++i) os << *i;
   return os;
}

istream &operator>>(istream &is, large &x)
{  char ch;
   x = 0;
   bool neg = 0;
   is >> ch;
   if (ch == '-')
   {  neg = 1;
      is.get(ch);
   }
   while (isdigit(ch))
   {  x = x * 10 + (ch - '0');
      is.get(ch);
   }
   if (neg) x = -x;
   is.putback(ch);
   return is;
}

large abs(large a)
{  if (a < 0) a = -a;
   return a;
}

large sqrt(const large &a)
{  large x = a, b = a, q;
   b <<= 1;
   while (b >>= 2, b > large(0)) x >>= 1;
   while (x > (q = a/x) + 1 || x < q - 1)
   {  x += q; x >>= 1;
   }
   return x < q ? x : q;
}

large power(large x, uint n)
{  large y=1;
   while (n)
   {  if (n & 1) y *= x;
      x *= x; n >>= 1;
   }
   return y;
}
